// Note that most declarations here are inherited, and so too are the javadocs.
// Since this is a proxy, there is no need to revise the interface.

public class EncryptingFileManager extends SimpleFileManager {
  // THIS IS AN EXAMPLE OF THE PROXY PATTERN.

  // AF(this) = A file manager with encrypted files to change stored in cache in
  //            the entries where value.dirty is true.
  // RI: No new invariants.

  /** A constant used for a simple encryption */
  private static final int KEY = 3;

  /**
   * @spec.effects Constucts an EncryptingFileManager with an empty cache.
   */
  public EncryptingFileManager() {
    super();
  }

  @Override
  public String get(String file) {
    checkRep();
    assert file != null;

    // If the file is in the cache, decode it and return it.
    if (cache.containsKey(file)) {
      try {
        return decode(cache.get(file).contents);
      } finally {
        checkRep();
      }
    }

    // Delegate the get operation to super
    return super.get(file);
  }

  @Override
  public void put(String file, String contents) {
    // Delegate the put operation to super
    super.put(file, encode(contents));
  }

  @Override
  public String cacheContent(String file) {
    return this.cache.getOrDefault(file, new FileData("", false)).contents;
  }

  /**
   * Given an encrypted String, returns the decrypted equivalent.
   *
   * @param encrypted the encrypted string
   * @return the decrypted equivalent of the string
   */
  private String decode(String encrypted) {
    // Decrypt the string by decrementing
    // each character by a constant amount (KEY).
    char[] decrypted = encrypted.toCharArray();
    for (int i = 0; i < decrypted.length; i++) {
      decrypted[i] -= KEY;
    }
    return new String(decrypted);
  }

  /**
   * Given an decrypted String, returns the encrypted equivalent.
   *
   * @param decrypted the decrypted string
   * @return the encrypted equivalent of the string
   */
  private String encode(String decrypted) {
    // Encrypt the string by incrementing
    // each character by a constant amount (KEY).
    char[] encrypted = decrypted.toCharArray();
    for (int i = 0; i < encrypted.length; i++) {
      encrypted[i] += KEY;
    }
    return new String(encrypted);
  }
}
