import java.io.*;

/**
 * Retrieves and writes file contents.  A FileManager does not write to files
 * immediately, but stores their intended state until prompted to update the
 * file system with changes.
 *
 * Additionally, a PrintingFileManager has a name, and prints some debugging
 * information on each operation.
 */
public class PrintingFileManager implements FileManager {
  // THIS IS AN EXAMPLE OF THE DECORATOR PATTERN.

  // AF(this) = A PrintingFileManager this.manager with the name this.name.
  // RI: manager, name != null

  /** The size of a read buffer. */
  private static final int BUFFER_LEN = 1024;

  /** The manager that this manager delegates operations to. */
  private FileManager manager;
  /** The name of this. */
  private String name;

  public PrintingFileManager(FileManager manager, String name) {
    this.manager = manager;
    this.name = name;

    checkRep();
  }

  @Override
  public String get(String file) {
    checkRep();

    // Delegate the get operation to this.manager.
    String res = this.manager.get(file);

    System.out.format("%s.get(%s);\n", this.name, file);
    System.out.println(res);
    System.out.println();

    checkRep();
    return res;
  }

  @Override
  public void put(String file, String contents) {
    checkRep();

    // Delegate the put operation to this.manager.
    this.manager.put(file, contents);

    System.out.format("%s.put(%s,\n%s\n);\n", this.name, file, contents);
    System.out.println();

    checkRep();
  }

  @Override
  public boolean update(String file) {
    checkRep();

    // Delegate the update operation to this.manager.
    boolean res = this.manager.update(file);

    System.out.format("%s.update(%s);\n", this.name, file);
    FileReader f;
    try {
      f = new FileReader(file);
      while (true) {
        char[] buf = new char[BUFFER_LEN];
        int readRes = f.read(buf, 0, BUFFER_LEN);

        if (readRes == -1) {
          break;
        }

        System.out.print(new String(buf, 0, readRes));
      }
      System.out.println();
    } catch(IOException e) {
      System.out.println("File open or write failed");
    }

    System.out.println();

    checkRep();
    return res;
  }

  /**
   * Checks that the representation invariant holds.
   */
  private void checkRep() {
    assert this.manager != null;
    assert this.name != null;
  }
}
